defmodule DecentralisedBookIndexWeb.PublisherLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  @impl true
  def render(assigns) do
    ~H"""
    <.header>
      Listing Publishers
      <:actions>
        <.link patch={~p"/publishers/new"}>
          <.button>New Publisher</.button>
        </.link>
      </:actions>
    </.header>

    <div class="pt-2">
      <.table
        id="publishers"
        rows={@streams.publishers}
        row_click={fn {_id, publisher} -> JS.navigate(~p"/publishers/#{publisher}") end}
      >
        <:col :let={{_id, publisher}} label="Name">{publisher.name}</:col>

        <:action :let={{_id, publisher}}>
          <div class="sr-only">
            <.link navigate={~p"/publishers/#{publisher}"}>Show</.link>
          </div>

          <.link patch={~p"/publishers/#{publisher}/edit"}>Edit</.link>
        </:action>
      </.table>
    </div>

    <.modal
      :if={@live_action in [:new, :edit]}
      id="publisher-modal"
      show
      on_cancel={JS.patch(~p"/publishers")}
    >
      <.live_component
        module={DecentralisedBookIndexWeb.PublisherLive.FormComponent}
        id={(@publisher && @publisher.id) || :new}
        title={@page_title}
        current_user={@current_user}
        action={@live_action}
        publisher={@publisher}
        patch={~p"/publishers"}
      />
    </.modal>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok,
     socket
     |> stream(
       :publishers,
       Ash.read!(DecentralisedBookIndex.Metadata.Publisher, actor: socket.assigns[:current_user])
     )
     |> assign_new(:current_user, fn -> nil end)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    {:noreply, apply_action(socket, socket.assigns.live_action, params)}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Publisher")
    |> assign(
      :publisher,
      Ash.get!(DecentralisedBookIndex.Metadata.Publisher, id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Publisher")
    |> assign(:publisher, nil)
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Publishers")
    |> assign(:publisher, nil)
  end

  @impl true
  def handle_info(
        {DecentralisedBookIndexWeb.PublisherLive.FormComponent, {:saved, publisher}},
        socket
      ) do
    {:noreply, stream_insert(socket, :publishers, publisher)}
  end

  @impl true
  def handle_event("delete", %{"id" => id}, socket) do
    publisher =
      Ash.get!(DecentralisedBookIndex.Metadata.Publisher, id, actor: socket.assigns.current_user)

    Ash.destroy!(publisher, actor: socket.assigns.current_user)

    {:noreply, stream_delete(socket, :publishers, publisher)}
  end
end
